"""Added createdDate to BranchIndex

Peek Plugin Database Migration Script

Revision ID: 361988cd327d
Revises: 8ef78f862ac8
Create Date: 2019-05-07 22:14:27.012735

"""

# revision identifiers, used by Alembic.
revision = "361988cd327d"
down_revision = "8ef78f862ac8"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "BranchIndex",
        sa.Column("createdDate", sa.DateTime(timezone=True), nullable=False),
        schema="pl_diagram",
    )
    op.add_column(
        "BranchIndex",
        sa.Column("updatedDate", sa.DateTime(timezone=True), nullable=True),
        schema="pl_diagram",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("BranchIndex", "updatedDate", schema="pl_diagram")
    op.drop_column("BranchIndex", "createdDate", schema="pl_diagram")
    # ### end Alembic commands ###



"""add text border for curved text

Peek Plugin Database Migration Script

Revision ID: 128b23798db0
Revises: e3c2c2580dea
Create Date: 2022-07-07 13:51:54.645086

"""

# revision identifiers, used by Alembic.
revision = "128b23798db0"
down_revision = "e3c2c2580dea"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    # add DispText.borderColorId and the foreign key
    op.add_column(
        "DispCurvedText",
        sa.Column("borderColorId", sa.Integer(), nullable=True),
        schema="pl_diagram",
    )
    op.create_foreign_key(
        "DispCurvedText_borderColorId_fkey",
        "DispCurvedText",
        "DispColor",
        ["borderColorId"],
        ["id"],
        source_schema="pl_diagram",
        referent_schema="pl_diagram",
    )


def downgrade():
    # drop DispText.borderColorId and the foreign key
    op.drop_constraint(
        "DispCurvedText_borderColorId_fkey",
        "DispCurvedText",
        schema="pl_diagram",
    )
    op.drop_column("DispCurvedText", "borderColorId", schema="pl_diagram")



from typing import List, Optional
from vortex.Tuple import Tuple, addTupleType, TupleField

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from peek_plugin_diagram.tuples.model.ImportLiveDbDispLinkTuple import (
    ImportLiveDbDispLinkTuple,
)
from peek_plugin_diagram.tuples.shapes.ImportDispPolylineTuple import (
    ImportDispPolylineTuple,
)


@addTupleType
class ImportDispGroupPtrTuple(Tuple):
    """Imported Display Text

    This tuple is used by other plugins to load TEXT objects into the diagram.

    """

    __tupleType__ = diagramTuplePrefix + "ImportDispGroupPtrTuple"

    ### BEGIN DISP COMMON FIELDS ###

    # The actions to perform when this line is clicked
    ACTION_NONE = ImportDispPolylineTuple.ACTION_NONE
    ACTION_POSITION_ON = ImportDispPolylineTuple.ACTION_POSITION_ON
    # For ACTION_POSITION_ON, Add the following to the display data
    # data['actionPos'] = {k='coordSetKey', x=x, y=y, z=zoom}

    #: Key, This value is a unique ID of the object that this graphic represents
    # It's used to link this graphical object to objects in other plugins, like vertices
    # in the peek-plugin-graphdb plugin.
    # Length = 50
    key: str = TupleField()

    #: Selectable, Is is this item selectable?, the layer also needs selectable=true
    selectable: bool = TupleField()

    #: Overlay, Is is this shape an overlay?, Overlays are sometimes used to add dynamic
    # data to the diagram, such as a Job, Operation, or placing a green box over a red
    # one to change it's state.
    overlay: bool = TupleField()

    #: Action, An action to perform when this display item is clicked.
    # See the ACTION_NONE constants for values.
    action: Optional[int] = TupleField(None)

    #: Data, Generic data, this is passed to the popup context in the UI.
    # peek_plugin_diagram doesn't care as long as it's json compatible or None
    # Json length Length = 400
    data: Optional[dict] = TupleField(None)

    #: The hash of the level to link to (Matches ImportDispLevel.importHash)
    # If the level shows at all zoom levels, then filtering will move on to the
    # individual disp level.
    levelHash: str = TupleField()

    #: The hash of the layer to link to (Matches ImportDispLayer.importHash)
    layerHash: str = TupleField()

    #: The Z Order of this display object when compared against other objects on
    # same layer and level.
    zOrder: int = TupleField()

    #: The unique hash of this display object
    importHash: str = TupleField()

    #: The unique hash for all the display items imported in a group with this one.
    #: for example, a page or tile reference.
    importGroupHash: str = TupleField()

    #: The key of the ModelSet to import into
    modelSetKey: str = TupleField()

    #: The Key of the Coordinate Set to import into
    coordSetKey: str = TupleField()

    #: Related links to LiveDB values for this display item
    liveDbDispLinks: List[ImportLiveDbDispLinkTuple] = TupleField()

    #: Parent DispGroup Hash, If this disp is part of a disp group then set this field to
    # the ImportDispGroupTuple.importHash fields value
    # NOTE: If this disp is part of a display group, then the GEOM coordinates need to
    # be relative to 0x0.
    # NOTE: Disps that are apart of a group must all be imported with the same
    # importGroupHash, during the same import call.
    parentDispGroupHash: str = TupleField()

    ### BEGIN FIELDS FOR THIS DISP ###

    #: The rotation of the text
    rotation: float = TupleField()

    #: Vertical Stretch, The scale to stretch the display group horizontally
    verticalScale: float = TupleField(1)

    #: Horizontal Stretch, The scale to stretch the display group vertically
    horizontalScale: float = TupleField(1)

    #: Disp Group Hash, The value of a "ImportDispGroupTuple.importHash" to point to
    # This can be null if only an instance (GroupPtr) is imported with no template (Group)
    targetDispGroupHash: Optional[str] = TupleField()

    #: The name of the template that this group pointer was created from.
    targetDispGroupName: Optional[str] = TupleField()

    #: The center location of this
    geom: List[float] = TupleField()



"""added location index

Peek Plugin Database Migration Script

Revision ID: 23117803d414
Revises: 1fe753e1f369
Create Date: 2017-09-09 16:10:10.582473

"""

# revision identifiers, used by Alembic.
revision = "23117803d414"
down_revision = "1fe753e1f369"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "LocationIndexCompiled",
        sa.Column("id", sa.Integer(), nullable=False, autoincrement=True),
        sa.Column("indexBucket", sa.String(length=100), nullable=False),
        sa.Column("blobData", sa.LargeBinary(), nullable=False),
        sa.Column("lastUpdate", sa.String(length=50), nullable=False),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["modelSetId"], ["pl_diagram.ModelSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id", "indexBucket"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DKIndexUpdate_coordSetId",
        "LocationIndexCompiled",
        ["modelSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DKIndexUpdate_gridKey",
        "LocationIndexCompiled",
        ["indexBucket"],
        unique=True,
        schema="pl_diagram",
    )
    op.create_table(
        "LocationIndexCompilerQueue",
        sa.Column("id", sa.Integer(), nullable=False, autoincrement=True),
        sa.Column("indexBucket", sa.String(length=100), nullable=False),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["modelSetId"], ["pl_diagram.ModelSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id", "indexBucket", "modelSetId"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DKCompQueue_coordSetId_gridKey",
        "LocationIndexCompilerQueue",
        ["modelSetId", "indexBucket"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_table(
        "LocationIndex",
        sa.Column("indexBucket", sa.String(length=100), nullable=False),
        sa.Column("dispId", sa.Integer(), nullable=False),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["dispId"], ["pl_diagram.DispBase.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(
            ["modelSetId"],
            ["pl_diagram.ModelSet.id"],
        ),
        sa.PrimaryKeyConstraint("indexBucket", "dispId"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_LocationIndex_dispId",
        "LocationIndex",
        ["dispId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_LocationIndex_indexBucket",
        "LocationIndex",
        ["indexBucket"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_LocationIndex_modelSetId",
        "LocationIndex",
        ["modelSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.add_column(
        "DispBase",
        sa.Column("locationJson", sa.String(length=120), nullable=True),
        schema="pl_diagram",
    )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("DispBase", "locationJson", schema="pl_diagram")
    op.drop_index(
        "idx_LocationIndex_modelSetId", table_name="LocationIndex", schema="pl_diagram"
    )
    op.drop_index(
        "idx_LocationIndex_indexBucket", table_name="LocationIndex", schema="pl_diagram"
    )
    op.drop_index(
        "idx_LocationIndex_dispId", table_name="LocationIndex", schema="pl_diagram"
    )
    op.drop_table("LocationIndex", schema="pl_diagram")
    op.drop_index(
        "idx_DKCompQueue_coordSetId_gridKey",
        table_name="LocationIndexCompilerQueue",
        schema="pl_diagram",
    )
    op.drop_table("LocationIndexCompilerQueue", schema="pl_diagram")
    op.drop_index(
        "idx_DKIndexUpdate_gridKey",
        table_name="LocationIndexCompiled",
        schema="pl_diagram",
    )
    op.drop_index(
        "idx_DKIndexUpdate_coordSetId",
        table_name="LocationIndexCompiled",
        schema="pl_diagram",
    )
    op.drop_table("LocationIndexCompiled", schema="pl_diagram")
    # ### end Alembic command



import logging

from sqlalchemy import Column, LargeBinary, BigInteger
from sqlalchemy import ForeignKey
from sqlalchemy import Integer, String
from sqlalchemy.orm import relationship
from sqlalchemy.sql.schema import Index
from vortex.Tuple import Tuple, addTupleType

from peek_abstract_chunked_index.private.tuples.ACIEncodedChunkTupleABC import (
    ACIEncodedChunkTupleABC,
)
from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from peek_plugin_diagram._private.storage.DeclarativeBase import DeclarativeBase
from peek_plugin_diagram._private.storage.ModelSet import ModelSet

logger = logging.getLogger(__name__)


@addTupleType
class BranchIndexEncodedChunk(Tuple, DeclarativeBase, ACIEncodedChunkTupleABC):
    __tablename__ = "BranchIndexEncodedChunk"
    __tupleType__ = diagramTuplePrefix + "BranchIndexEncodedChunkTable"

    id = Column(BigInteger, primary_key=True, autoincrement=True)

    modelSetId = Column(
        Integer, ForeignKey("ModelSet.id", ondelete="CASCADE"), nullable=False
    )
    modelSet = relationship(ModelSet)

    chunkKey = Column(String, nullable=False)
    encodedData = Column(LargeBinary, nullable=False)
    encodedHash = Column(String, nullable=False)
    lastUpdate = Column(String, nullable=False)

    __table_args__ = (
        Index(
            "idx_BranchIndexEnc_modelSetId_chunkKey",
            modelSetId,
            chunkKey,
            unique=False,
        ),
    )

    @property
    def ckiChunkKey(self):
        return self.chunkKey

    @property
    def ckiEncodedData(self):
        return self.encodedData

    @property
    def ckiHasEncodedData(self) -> bool:
        return bool(self.encodedData)

    @property
    def ckiLastUpdate(self):
        return self.lastUpdate

    @classmethod
    def ckiCreateDeleteEncodedChunk(cls, chunkKey: str):
        return BranchIndexEncodedChunk(indexBucket=chunkKey)

    @classmethod
    def sqlCoreChunkKeyColumn(cls):
        return cls.__table__.c.chunkKey

    @classmethod
    def sqlCoreLastUpdateColumn(cls):
        return cls.__table__.c.lastUpdate

    @classmethod
    def sqlCoreLoad(cls, row):
        return BranchIndexEncodedChunk(
            id=row.id,
            modelSetId=row.modelSetId,
            chunkKey=row.chunkKey,
            encodedData=row.encodedData,
            encodedHash=row.encodedHash,
            lastUpdate=row.lastUpdate,
        )



import logging
from typing import List

from twisted.internet import reactor
from twisted.internet.defer import Deferred
from twisted.internet.defer import inlineCallbacks
from vortex.Payload import Payload

from peek_plugin_base.storage.DbConnection import DbSessionCreator
from peek_plugin_base.storage.RunPyInPg import runPyInPg
from peek_plugin_diagram._private.storage.Display import DispBase
from peek_plugin_diagram.tuples.ImportGroupHashTuple import (
    ImportGroupHashTuple,
)
from peek_plugin_diagram._private.worker.tasks.ImportDispTask import (
    importDispsTask,
)
from peek_plugin_livedb.server.LiveDBWriteApiABC import LiveDBWriteApiABC

logger = logging.getLogger(__name__)


class SqlController:
    @classmethod
    @inlineCallbacks
    def getImportGroupHashes(
        cls,
        dbSessionCreator: DbSessionCreator,
        modelSetKey: str,
        coordSetKey: str,
        importGroupHashContains: str,
    ) -> List[str]:
        ret = yield runPyInPg(
            logger,
            dbSessionCreator,
            cls._getImportGroupHashes,
            None,
            modelSetKey=modelSetKey,
            coordSetKey=coordSetKey,
            importGroupHashContains=importGroupHashContains,
        )
        return ret

    @classmethod
    def _getImportGroupHashes(
        self,
        plpy,
        modelSetKey: str,
        coordSetKey: str,
        importGroupHashContains: str,
    ) -> List[ImportGroupHashTuple]:
        rows = plpy.execute(
            f"""
            SELECT DISTINCT
                db."importGroupHash"
            FROM
                pl_diagram."DispBase" AS db
                JOIN pl_diagram."ModelCoordSet" AS mcs ON mcs.id = db."coordSetId"
                JOIN pl_diagram."ModelSet" AS ms ON ms.id = mcs."modelSetId"
            WHERE
                ms.key = '{modelSetKey}'
                AND mcs.key = '{coordSetKey}'
                AND db."importGroupHash" LIKE '%{importGroupHashContains}%';
            """
        )

        ret = []
        for row in rows:
            tuple_ = ImportGroupHashTuple()
            tuple_.fromDict(row)
            ret.append(tuple_)
        return ret


class DispImportController:
    def __init__(
        self,
        dbSessionCreator: DbSessionCreator,
        liveDbWriteApi: LiveDBWriteApiABC,
    ):
        self._dbSessionCreator = dbSessionCreator
        self._liveDbWriteApi = liveDbWriteApi

    def shutdown(self):
        self._liveDbWriteApi = None

    @inlineCallbacks
    def importDisps(
        self,
        modelSetKey: str,
        coordSetKey: str,
        importGroupHash: str,
        dispsEncodedPayload: bytes,
    ):
        liveDbItemsToImport = yield importDispsTask.delay(
            modelSetKey, coordSetKey, importGroupHash, dispsEncodedPayload
        )

        if liveDbItemsToImport:
            yield self._liveDbWriteApi.importLiveDbItems(
                modelSetKey, liveDbItemsToImport
            )

            # Give and connector plugins time to load the new items
            yield self._sleep(2.0)

            yield self._liveDbWriteApi.pollLiveDbValueAcquisition(
                modelSetKey, [i.key for i in liveDbItemsToImport]
            )

    def _sleep(self, seconds):
        d = Deferred()
        reactor.callLater(seconds, d.callback, True)
        return d

    def getImportGroupHashes(
        self, modelSetKey: str, coordSetKey: str, importGroupHashContains: str
    ):
        return SqlController.getImportGroupHashes(
            self._dbSessionCreator,
            modelSetKey,
            coordSetKey,
            importGroupHashContains,
        )

    def removeDispsByImportGroupHash(
        self, modelSetKey: str, coordSetKey: str, importGroupHash: str
    ):
        emptyPayload = Payload(tuples=[]).toEncodedPayload()
        return importDispsTask.delay(
            modelSetKey, coordSetKey, importGroupHash, emptyPayload
        )



from typing import Optional

from vortex.Tuple import Tuple, addTupleType, TupleField

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from peek_plugin_diagram.tuples.shapes.ImportDispPolylineTuple import (
    ImportDispPolylineTuple,
)


@addTupleType
class ImportDispGroupTuple(Tuple):
    """Imported Display Group

    This tuple is used by other plugins to load groups of display objects
    into the diagram model.

    Display items are apart of this group if their :code:`parentDispGroupHash` value is
    set to the :code:`importHash` of this group.

    The display items that are apart of this group will never be displayed.

    To create an instance of this display group on the diagram,
    import a ImportDispGroupPtyTuple display object.

    """

    __tupleType__ = diagramTuplePrefix + "ImportDispGroupTuple"

    ### BEGIN DISP COMMON FIELDS ###

    # The actions to perform when this line is clicked
    ACTION_NONE = ImportDispPolylineTuple.ACTION_NONE
    ACTION_POSITION_ON = ImportDispPolylineTuple.ACTION_POSITION_ON
    # For ACTION_POSITION_ON, Add the following to the display data
    # data['actionPos'] = {k='coordSetKey', x=x, y=y, z=zoom}

    #: Selectable, Is is this item selectable?, the layer also needs selectable=true
    selectable: bool = TupleField()

    #: Overlay, Is is this shape an overlay?, Overlays are sometimes used to add dynamic
    # data to the diagram, such as a Job, Operation, or placing a green box over a red
    # one to change it's state.
    overlay: bool = TupleField()

    #: Action, An action to perform when this display item is clicked.
    # See the ACTION_NONE constants for values.
    action: Optional[int] = TupleField(None)

    #: Data, Generic data, this is passed to the popup context in the UI.
    # peek_plugin_diagram doesn't care as long as it's json compatible or None
    # Json length Length = 400
    data: Optional[dict] = TupleField(None)

    #: The unique hash of this display object
    # This will be referenced by:
    # ImportDispGroupPrtTuple.targetDispGroupHash, and
    # Any disps in the group, disp.parentDispGroupHash
    importHash: str = TupleField()

    #: The unique hash for all the display items imported in a group with this one.
    #: for example, a page or tile reference.
    importGroupHash: str = TupleField()

    #: The key of the ModelSet to import into
    modelSetKey: str = TupleField()

    #: The Key of the Coordinate Set to import into
    coordSetKey: str = TupleField()

    ### BEGIN FIELDS FOR THIS DISP ###

    #: A name for this dispGroup
    name: str = TupleField()

    # If this is set to true, then this group will be available in the user interface
    # for edit support (located in the special '##|dispgroup' grid key)
    compileAsTemplate: bool = TupleField(False)



from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from vortex.Tuple import Tuple, addTupleType, TupleField


@addTupleType
class ImportLiveDbDispLinkTuple(Tuple):
    """Imported LiveDB Display Link

    A LiveDB value is a value that changes based on telemetry or other updates.
    This in turn drives an update to an attribute to a primitive display object.

    """

    __tupleType__ = diagramTuplePrefix + "ImportLiveDbDispLinkTuple"

    DISP_ATTR_FILL_COLOR = "fillColorId"
    DISP_ATTR_BORDER_COLOR = "borderColorId"
    DISP_ATTR_LINE_COLOR = "lineColorId"
    DISP_ATTR_EDGE_COLOR = "edgeColorId"
    DISP_ATTR_COLOR = "colorId"
    DISP_ATTR_LINE_STYLE = "lineStyleId"
    DISP_ATTR_LINE_WIDTH = "lineWidth"
    DISP_ATTR_TEXT = "text"
    DISP_ATTR_TEXT_STYLE = "textStyleId"
    DISP_ATTR_FILL_PERCENT = "fillPercent"

    #:  The attribute name of the display object that this live db key updates
    dispAttrName: str = TupleField()

    #:  The key is the name of the value in the source system.
    liveDbKey: str = TupleField()

    #: The unique hash of this livedb object
    # importKeyHash: str = TupleField()

    #: The unique hash of the display object this link relates to
    importDispHash: str = TupleField()

    #: The unique hash for all the display items imported in a group with this one.
    #: for example, a page or tile reference.
    importGroupHash: str = TupleField()

    modelSetKey: str = TupleField()
    coordSetKey: str = TupleField()

    #:  Additional data for this object
    props: dict = TupleField()

    #: This is used internally for the import, DO NOT USE
    internalDispId: int = TupleField()

    #: This is used internally for the import, DO NOT USE
    internalRawValue: str = TupleField()

    #: This is used internally for the import, DO NOT USE
    internalDisplayValue: str = TupleField()



"""Increased size of disp.json

Peek Plugin Database Migration Script

Revision ID: cbd5e370686f
Revises: 9e7ca1acd6be
Create Date: 2017-08-12 17:34:00.532517

"""

# revision identifiers, used by Alembic.
revision = "cbd5e370686f"
down_revision = "9e7ca1acd6be"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.alter_column(
        "DispBase",
        "dispJson",
        existing_type=sa.VARCHAR(length=200000),
        nullable=True,
        schema="pl_diagram",
    )

    op.alter_column(
        "DispPolygon",
        "geomJson",
        existing_type=sa.VARCHAR(length=200000),
        nullable=False,
        schema="pl_diagram",
    )

    op.alter_column(
        "DispPolyline",
        "geomJson",
        existing_type=sa.VARCHAR(length=200000),
        nullable=False,
        schema="pl_diagram",
    )


def downgrade():
    raise Exception("Downgrade is not implemented")



"""rebuild location index

Peek Plugin Database Migration Script

Revision ID: 34dd05f474df
Revises: 298b291aac2d
Create Date: 2017-12-29 20:00:27.258810

"""

# revision identifiers, used by Alembic.
revision = "34dd05f474df"
down_revision = "298b291aac2d"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    op.execute(
        """
        TRUNCATE TABLE pl_diagram."LocationIndexCompiled" CASCADE
    """
    )

    op.execute(
        """
        INSERT INTO pl_diagram."LocationIndexCompilerQueue"
        ("modelSetId", "indexBucket")
        SELECT "modelSetId", "indexBucket"
        FROM pl_diagram."LocationIndex"
    """
    )


def downgrade():
    pass



"""added compiler q indexes

Peek Plugin Database Migration Script

Revision ID: 2fc6d3246717
Revises: 1856b7fb8803
Create Date: 2019-10-11 15:19:48.918248

"""

# revision identifiers, used by Alembic.
revision = "2fc6d3246717"
down_revision = "1856b7fb8803"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index(
        "idx_BICompQueue_modelSetId_chunkKey",
        "BranchIndexCompilerQueue",
        ["modelSetId", "chunkKey"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispCompQueue_dispId",
        "DispCompilerQueue",
        ["dispId"],
        unique=False,
        schema="pl_diagram",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(
        "idx_DispCompQueue_dispId", table_name="DispCompilerQueue", schema="pl_diagram"
    )
    op.drop_index(
        "idx_BICompQueue_modelSetId_chunkKey",
        table_name="BranchIndexCompilerQueue",
        schema="pl_diagram",
    )
    # ### end Alembic commands ###



import logging
from typing import Dict

from peek_abstract_chunked_index.private.server.client_handlers.ACIChunkUpdateHandlerABC import (
    ACIChunkUpdateHandlerABC,
)
from peek_abstract_chunked_index.private.tuples.ACIEncodedChunkTupleABC import (
    ACIEncodedChunkTupleABC,
)
from peek_plugin_diagram._private.client.controller.GridCacheController import (
    clientGridUpdateFromServerFilt,
)
from peek_plugin_diagram._private.storage.GridKeyIndex import (
    GridKeyIndexCompiled,
)

logger = logging.getLogger(__name__)


class ClientGridUpdateHandler(ACIChunkUpdateHandlerABC):
    _ChunkedTuple: ACIEncodedChunkTupleABC = GridKeyIndexCompiled
    _updateFromLogicFilt: Dict = clientGridUpdateFromServerFilt
    _logger: logging.Logger = logger



"""DispText.textStyleId not null

Peek Plugin Database Migration Script

Revision ID: 1fe753e1f369
Revises: bfdd4f46e293
Create Date: 2017-08-26 17:46:15.455355

"""

# revision identifiers, used by Alembic.
revision = "1fe753e1f369"
down_revision = "bfdd4f46e293"
branch_labels = None
depends_on = None

import sqlalchemy as sa
from alembic import op


def upgrade():
    op.drop_index("idx_DispText_styleId", table_name="DispText", schema="pl_diagram")

    op.alter_column(
        "DispText", "textStyleId", type_=sa.Integer, nullable=False, schema="pl_diagram"
    )
    op.create_index(
        "idx_DispText_styleId",
        "DispText",
        ["textStyleId"],
        unique=False,
        schema="pl_diagram",
    )


def downgrade():
    op.alter_column(
        "DispText", "textStyleId", type_=sa.Integer, nullable=True, schema="pl_diagram"
    )



import logging
from typing import Union

from twisted.internet.defer import Deferred
from twisted.internet.defer import inlineCallbacks
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

from peek_plugin_diagram._private.client.controller.LookupCacheController import (
    LookupCacheController,
)

logger = logging.getLogger(__name__)


class ClientLookupTupleProvider(TuplesProviderABC):
    def __init__(self, lookupCacheController: LookupCacheController):
        self._lookupCacheController = lookupCacheController

    @inlineCallbacks
    def makeVortexMsg(
        self, filt: dict, tupleSelector: TupleSelector
    ) -> Union[Deferred, bytes]:
        tuples = self._lookupCacheController.lookups(tupleSelector.name)

        payloadEnvelope = yield Payload(
            filt, tuples=tuples
        ).makePayloadEnvelopeDefer()
        vortexMsg = yield payloadEnvelope.toVortexMsgDefer()
        return vortexMsg



from typing import Dict
from typing import List

from vortex.Tuple import Tuple
from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix


@addTupleType
class ImportGroupHashTuple(Tuple):
    __tupleType__ = diagramTuplePrefix + "ImportGroupHashTuple"

    importGroupHash: List[str] = TupleField()

    def fromDict(self, d: Dict):
        importGroupHash = d.get("importGroupHash")
        if importGroupHash:
            self.importGroupHash = importGroupHash



import logging
from typing import Dict

from sqlalchemy import select

from peek_plugin_diagram._private.storage.Display import DispColor
from peek_plugin_diagram._private.storage.Display import DispLayer
from peek_plugin_diagram._private.storage.Display import DispLevel
from peek_plugin_diagram._private.storage.Display import DispLineStyle
from peek_plugin_diagram._private.storage.Display import DispTextStyle
from peek_plugin_livedb.tuples.ImportLiveDbItemTuple import (
    ImportLiveDbItemTuple,
)

logger = logging.getLogger(__name__)

textTable = DispTextStyle.__table__
lineTable = DispLineStyle.__table__
colorTable = DispColor.__table__
levelTable = DispLevel.__table__
layerTable = DispLayer.__table__


class LiveDbDisplayValueConverter:
    _colorIdByImportHash = None
    _lineStyleIdByImportHash = None

    @staticmethod
    def create(ormSession, modelSetId: int) -> "LiveDbDisplayValueConverter":
        self = LiveDbDisplayValueConverter()

        # self._textStyleIdByImportHash = self._loadLookupByModelSet(
        #     ormSessionCreator, modelSetId, textTable
        # )

        self._lineStyleIdByImportHash = self._loadLookupByModelSet(
            ormSession, modelSetId, lineTable
        )

        self._colorIdByImportHash = self._loadLookupByModelSet(
            ormSession, modelSetId, colorTable
        )

        # self._layerByImportHash = self._loadLookupByModelSet(
        #     ormSessionCreator, modelSetId, layerTable
        # )
        #
        return self

    def translate(self, dataType, rawValue):
        return self._liveDbTranslators[dataType](self, rawValue)

    @staticmethod
    def _loadLookupByModelSet(
        ormSession, modelSetId: int, table
    ) -> Dict[str, int]:
        resultSet = ormSession.execute(
            select([table.c.importHash, table.c.id]).where(
                table.c.modelSetId == modelSetId
            )
        )

        return dict(resultSet.fetchall())

    def _liveDbValueTranslateColorId(self, value):
        return self._colorIdByImportHash.get(value)

    def _liveDbValueTranslateLineStyleId(self, value):
        return self._lineStyleIdByImportHash.get(value)

    def _liveDbValueTranslateLineWidth(self, value):
        return value

    def _liveDbValueTranslateText(self, value):
        return "" if value is None else value

    def _liveDbValueTranslateNumber(self, value):
        return value

    def _liveDbValueTranslateGroupId(self, value):
        raise NotImplementedError()

    _liveDbTranslators = {
        ImportLiveDbItemTuple.DATA_TYPE_COLOR: _liveDbValueTranslateColorId,
        ImportLiveDbItemTuple.DATA_TYPE_LINE_STYLE: _liveDbValueTranslateLineStyleId,
        ImportLiveDbItemTuple.DATA_TYPE_LINE_WIDTH: _liveDbValueTranslateLineWidth,
        ImportLiveDbItemTuple.DATA_TYPE_STRING_VALUE: _liveDbValueTranslateText,
        ImportLiveDbItemTuple.DATA_TYPE_NUMBER_VALUE: _liveDbValueTranslateNumber,
        ImportLiveDbItemTuple.DATA_TYPE_GROUP_PTR: _liveDbValueTranslateGroupId,
    }



"""Added CoordSet GridSizes

Peek Plugin Database Migration Script

Revision ID: 6a97dd344aed
Revises: cbd5e370686f
Create Date: 2017-08-13 13:17:24.923154

"""

# revision identifiers, used by Alembic.
revision = "6a97dd344aed"
down_revision = "cbd5e370686f"
branch_labels = None
depends_on = None

import sqlalchemy as sa
from alembic import op


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "ModelCoordSetGridSize",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("key", sa.Integer(), nullable=False),
        sa.Column("min", sa.Float(), nullable=False),
        sa.Column("max", sa.Float(), nullable=False),
        sa.Column("xGrid", sa.Integer(), nullable=False),
        sa.Column("yGrid", sa.Integer(), nullable=False),
        sa.Column("coordSetId", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["coordSetId"], ["pl_diagram.ModelCoordSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_CoordSetGridSize_key",
        "ModelCoordSetGridSize",
        ["coordSetId", "key"],
        unique=True,
        schema="pl_diagram",
    )

    op.add_column(
        "ModelCoordSet",
        sa.Column("smallestTextSize", sa.Float(), nullable=True, server_default="6.0"),
        schema="pl_diagram",
    )

    op.add_column(
        "ModelCoordSet",
        sa.Column("smallestShapeSize", sa.Float(), nullable=True, server_default="2.0"),
        schema="pl_diagram",
    )

    op.add_column(
        "ModelCoordSet",
        sa.Column("minZoom", sa.Float(), nullable=True, server_default="0.01"),
        schema="pl_diagram",
    )

    op.add_column(
        "ModelCoordSet",
        sa.Column("maxZoom", sa.Float(), nullable=True, server_default="10.0"),
        schema="pl_diagram",
    )

    op.execute(
        """
    UPDATE "pl_diagram"."ModelCoordSet"
    SET "smallestShapeSize" = 2.0,
        "smallestTextSize" = 6.0,
        "minZoom" = 0.01,
        "maxZoom" = 10.0
    """
    )

    op.alter_column(
        "ModelCoordSet",
        "smallestTextSize",
        existing_type=sa.Float(),
        nullable=False,
        schema="pl_diagram",
    )

    op.alter_column(
        "ModelCoordSet",
        "smallestShapeSize",
        existing_type=sa.Float(),
        nullable=False,
        schema="pl_diagram",
    )

    op.alter_column(
        "ModelCoordSet",
        "minZoom",
        existing_type=sa.Float(),
        nullable=False,
        schema="pl_diagram",
    )

    op.alter_column(
        "ModelCoordSet",
        "maxZoom",
        existing_type=sa.Float(),
        nullable=False,
        schema="pl_diagram",
    )

    op.execute(
        """
        INSERT INTO "pl_diagram"."ModelCoordSetGridSize"
        ("coordSetId", "key", "min", "max", "xGrid", "yGrid")
        SELECT "id", 0 as "key", 0 as "min", 0.04 as "max", 30000 as "xGrid", 30000 as "yGrid"
        FROM "pl_diagram"."ModelCoordSet"
    """
    )

    op.execute(
        """
        INSERT INTO "pl_diagram"."ModelCoordSetGridSize"
        ("coordSetId", "key", "min", "max", "xGrid", "yGrid")
        SELECT "id", 1 as "key", 0.04 as "min", 0.1 as "max", 10000 as "xGrid", 10000 as "yGrid"
        FROM "pl_diagram"."ModelCoordSet"
    """
    )

    op.execute(
        """
        INSERT INTO "pl_diagram"."ModelCoordSetGridSize"
        ("coordSetId", "key", "min", "max", "xGrid", "yGrid")
        SELECT "id", 2 as "key", 0.1 as "min", 0.5 as "max", 2000 as "xGrid", 2000 as "yGrid"
        FROM "pl_diagram"."ModelCoordSet"
    """
    )

    op.execute(
        """
        INSERT INTO "pl_diagram"."ModelCoordSetGridSize"
        ("coordSetId", "key", "min", "max", "xGrid", "yGrid")
        SELECT "id", 3 as "key", 0.5 as "min", 1000.0 as "max", 1000 as "xGrid", 1000 as "yGrid"
        FROM "pl_diagram"."ModelCoordSet"
    """
    )


def downgrade():
    raise NotImplementedError("Downgrade is not implemented")



"""removed string field sizes

Peek Plugin Database Migration Script

Revision ID: 1b17ef0cca17
Revises: f76791d27f6d
Create Date: 2019-02-15 22:54:38.653794

"""

# revision identifiers, used by Alembic.
revision = "1b17ef0cca17"
down_revision = "f76791d27f6d"
branch_labels = None
depends_on = None

import sqlalchemy as sa
from alembic import op


def upgrade():
    op.alter_column(
        "DispBase", "dispJson", type_=sa.String(), nullable=True, schema="pl_diagram"
    )
    op.alter_column(
        "DispBase",
        "locationJson",
        type_=sa.String(),
        nullable=True,
        schema="pl_diagram",
    )
    op.alter_column(
        "DispBase", "key", type_=sa.String(), nullable=True, schema="pl_diagram"
    )
    op.alter_column(
        "DispBase", "dataJson", type_=sa.String(), nullable=True, schema="pl_diagram"
    )

    op.alter_column(
        "DispGroup", "name", type_=sa.String(), nullable=False, schema="pl_diagram"
    )
    op.alter_column(
        "DispText", "geomJson", type_=sa.String(), nullable=False, schema="pl_diagram"
    )
    op.alter_column(
        "DispPolygon",
        "geomJson",
        type_=sa.String(),
        nullable=False,
        schema="pl_diagram",
    )
    op.alter_column(
        "DispPolyline",
        "geomJson",
        type_=sa.String(),
        nullable=False,
        schema="pl_diagram",
    )
    op.alter_column(
        "DispEllipse",
        "geomJson",
        type_=sa.String(),
        nullable=False,
        schema="pl_diagram",
    )
    op.alter_column(
        "DispGroupPointer",
        "geomJson",
        type_=sa.String(),
        nullable=False,
        schema="pl_diagram",
    )


def downgrade():
    op.alter_column(
        "DispBase",
        "dispJson",
        type_=sa.String(length=200000),
        nullable=True,
        schema="pl_diagram",
    )
    op.alter_column(
        "DispBase",
        "locationJson",
        type_=sa.String(length=120),
        nullable=True,
        schema="pl_diagram",
    )
    op.alter_column(
        "DispBase",
        "key",
        type_=sa.String(length=50),
        nullable=True,
        schema="pl_diagram",
    )
    op.alter_column(
        "DispBase",
        "dataJson",
        type_=sa.String(length=500),
        nullable=True,
        schema="pl_diagram",
    )

    op.alter_column(
        "DispGroup",
        "name",
        type_=sa.String(length=50),
        nullable=False,
        schema="pl_diagram",
    )
    op.alter_column(
        "DispText",
        "geomJson",
        type_=sa.String(length=2000),
        nullable=False,
        schema="pl_diagram",
    )
    op.alter_column(
        "DispPolygon",
        "geomJson",
        type_=sa.String(length=200000),
        nullable=False,
        schema="pl_diagram",
    )
    op.alter_column(
        "DispPolyline",
        "geomJson",
        type_=sa.String(length=200000),
        nullable=False,
        schema="pl_diagram",
    )
    op.alter_column(
        "DispEllipse",
        "geomJson",
        type_=sa.String(length=2000),
        nullable=False,
        schema="pl_diagram",
    )
    op.alter_column(
        "DispGroupPointer",
        "geomJson",
        type_=sa.String(length=2000),
        nullable=False,
        schema="pl_diagram",
    )



"""added scalable to lookup linestyle

Peek Plugin Database Migration Script

Revision ID: 8b8ebbe5ec7b
Revises: 42a8dc25e588
Create Date: 2022-05-26 14:33:55.123311

"""

# revision identifiers, used by Alembic.
from sqlalchemy import Boolean

revision = "8b8ebbe5ec7b"
down_revision = "42a8dc25e588"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column(
        "DispLineStyle",
        sa.Column("scalable", Boolean, server_default="false"),
        schema="pl_diagram",
    )


def downgrade():
    op.drop_column("DispLineStyle", "scalable", schema="pl_diagram")



"""added edgeTemplatesEnabled

Peek Plugin Database Migration Script

Revision ID: 1856b7fb8803
Revises: a86e0317e22a
Create Date: 2019-09-30 16:59:52.193761

"""

# revision identifiers, used by Alembic.
revision = "1856b7fb8803"
down_revision = "a86e0317e22a"
branch_labels = None
depends_on = None

import sqlalchemy as sa
from alembic import op


def upgrade():
    op.add_column(
        "ModelCoordSet",
        sa.Column(
            "edgeTemplatesEnabled", sa.Boolean(), nullable=True, server_default="false"
        ),
        schema="pl_diagram",
    )
    op.execute(
        """ UPDATE "pl_diagram"."ModelCoordSet"
                   SET "edgeTemplatesEnabled" = false """
    )
    op.alter_column(
        "ModelCoordSet",
        "edgeTemplatesEnabled",
        type_=sa.Boolean(),
        nullable=False,
        schema="pl_diagram",
    )


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("ModelCoordSet", "edgeTemplatesEnabled", schema="pl_diagram")
    # ### end Alembic commands ###



import logging
from pathlib import Path
from typing import Dict

from twisted.internet.defer import Deferred
from vortex.DeferUtil import vortexLogFailure
from vortex.PayloadEndpoint import PayloadEndpoint

from peek_abstract_chunked_index.private.client.controller.ACICacheControllerABC import (
    ACICacheControllerABC,
)
from peek_plugin_diagram._private.PluginNames import diagramFilt
from peek_plugin_diagram._private.server.client_handlers.ClientGridLoaderRpc import (
    ClientGridLoaderRpc,
)
from peek_plugin_diagram._private.tuples.grid.EncodedGridTuple import (
    EncodedGridTuple,
)
from peek_plugin_diagram._private.tuples.grid.GridTuple import GridTuple
from peek_plugin_diagram._private.tuples.grid.GridUpdateDateTuple import (
    GridUpdateDateTuple,
)

logger = logging.getLogger(__name__)

clientGridUpdateFromServerFilt = dict(key="clientGridUpdateFromServer")
clientGridUpdateFromServerFilt.update(diagramFilt)

clientCoordSetUpdateFromServerFilt = dict(key="clientCoordSetUpdateFromServer")
clientCoordSetUpdateFromServerFilt.update(diagramFilt)


class GridCacheController(ACICacheControllerABC):
    """Grid Cache Controller

    The grid cache controller stores all the grids in memory, allowing fast access from
    the mobile and desktop devices.

    NOTE: The grid set endpoint triggers a reload, this is in the case when grid sets
    are enable or disabled. Perhaps the system should just be restarted instead.

    """

    _ChunkedTuple = EncodedGridTuple
    _UpdateDateTupleABC = GridUpdateDateTuple
    _chunkLoadRpcMethod = ClientGridLoaderRpc.loadGrids
    _chunkIndexDeltaRpcMethod = ClientGridLoaderRpc.loadGridsIndexDelta
    _updateFromServerFilt = clientGridUpdateFromServerFilt
    _logger = logger

    #: This stores the cache of grid data for the clients
    _cache: Dict[str, GridTuple] = None

    _LOAD_CHUNK_SIZE = 75
    _LOAD_CHUNK_PARALLELISM = 4

    def __init__(self, clientId: str, pluginDataDir: Path):
        ACICacheControllerABC.__init__(self, clientId, pluginDataDir)

        self._coordSetEndpoint = PayloadEndpoint(
            clientCoordSetUpdateFromServerFilt, self._processCoordSetPayload
        )

    def _processCoordSetPayload(self, *args, **kwargs):
        d: Deferred = self.reloadCache()
        d.addErrback(vortexLogFailure, logger, consumeError=True)

    def shutdown(self):
        ACICacheControllerABC.shutdown(self)

        self._coordSetEndpoint.shutdown()
        self._coordSetEndpoint = None



from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from vortex.Tuple import Tuple, addTupleType, TupleField


@addTupleType
class DiagramCoordSetTuple(Tuple):
    """Diagram Coordinate Set Tuple

    This tuple represents a coordinate set in the diagram data storage.

    """

    __tupleType__ = diagramTuplePrefix + "DiagramCoordSetTuple"

    #:  The ID of the coordinate set.
    key: int = TupleField()

    #:  The name of the coordinate set.
    name: str = TupleField()



import logging
from typing import Union

from twisted.internet.defer import Deferred
from twisted.internet.defer import inlineCallbacks
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

from peek_plugin_diagram._private.client.controller.CoordSetCacheController import (
    CoordSetCacheController,
)

logger = logging.getLogger(__name__)


class ClientCoordSetTupleProvider(TuplesProviderABC):
    def __init__(self, coordSetCacheController: CoordSetCacheController):
        self._coordSetCacheController = coordSetCacheController

    @inlineCallbacks
    def makeVortexMsg(
        self, filt: dict, tupleSelector: TupleSelector
    ) -> Union[Deferred, bytes]:
        tuples = self._coordSetCacheController.coordSets

        payloadEnvelope = yield Payload(
            filt, tuples=tuples
        ).makePayloadEnvelopeDefer()
        vortexMsg = yield payloadEnvelope.toVortexMsgDefer()
        return vortexMsg



import logging
from typing import Union

from twisted.internet.defer import Deferred
from twisted.internet.defer import inlineCallbacks
from vortex.Payload import Payload
from vortex.PayloadEnvelope import PayloadEnvelope
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

from peek_plugin_diagram._private.client.controller.GridCacheController import (
    GridCacheController,
)

logger = logging.getLogger(__name__)


class GridCacheIndexTupleProvider(TuplesProviderABC):
    def __init__(self, gridCacheController: GridCacheController):
        self._gridCacheController = gridCacheController

    @inlineCallbacks
    def makeVortexMsg(
        self, filt: dict, tupleSelector: TupleSelector
    ) -> Union[Deferred, bytes]:

        index = tupleSelector.selector.get("index")

        if index is None:
            # Make it backwards compatible with v3.3.0 and below
            start = tupleSelector.selector.get("start")
            count = tupleSelector.selector.get("count")

            if count is None or count != 5000:
                return PayloadEnvelope(
                    filt, encodedPayload=Payload().toEncodedPayload()
                ).toVortexMsg()

            index = start / count

        encodedPayload = (
            self._gridCacheController.offlineUpdateDateByChunkKeyPayload(index)
        )
        payloadEnvelope = PayloadEnvelope(filt, encodedPayload=encodedPayload)
        vortexMsg = yield payloadEnvelope.toVortexMsgDefer()
        return vortexMsg
